/// <reference path="../../typings/index.d.ts" />
"use strict";
const NodeHTTP = require("http");
const NodeURL = require("url");
const async = require("async");
const index_1 = require("../utils/index");
(function (Status) {
    Status[Status["IDLE"] = 0] = "IDLE";
    Status[Status["STARTING"] = 1] = "STARTING";
    Status[Status["WORKING"] = 2] = "WORKING";
    Status[Status["CLOSING"] = 3] = "CLOSING";
    Status[Status["CLOSED"] = 4] = "CLOSED";
    Status[Status["ERROR"] = 5] = "ERROR";
})(exports.Status || (exports.Status = {}));
var Status = exports.Status;
(function (StatusCode) {
    StatusCode[StatusCode["CONTINUE"] = 100] = "CONTINUE";
    StatusCode[StatusCode["SWITCHING_PROTOCOLS"] = 101] = "SWITCHING_PROTOCOLS";
    StatusCode[StatusCode["PROCESSING"] = 102] = "PROCESSING";
    StatusCode[StatusCode["OK"] = 200] = "OK";
    StatusCode[StatusCode["CREATED"] = 201] = "CREATED";
    StatusCode[StatusCode["ACCEPTED"] = 202] = "ACCEPTED";
    StatusCode[StatusCode["NON_AUTHORITATIVE_INFORMATION"] = 203] = "NON_AUTHORITATIVE_INFORMATION";
    StatusCode[StatusCode["NO_CONTENT"] = 204] = "NO_CONTENT";
    StatusCode[StatusCode["RESET_CONTENT"] = 205] = "RESET_CONTENT";
    StatusCode[StatusCode["PARTIAL_CONTENT"] = 206] = "PARTIAL_CONTENT";
    StatusCode[StatusCode["MULTIPLE_CHOICES"] = 300] = "MULTIPLE_CHOICES";
    StatusCode[StatusCode["MOVED_PERMANENTLY"] = 301] = "MOVED_PERMANENTLY";
    StatusCode[StatusCode["MOVE_TEMPORARILY"] = 302] = "MOVE_TEMPORARILY";
    StatusCode[StatusCode["SEE_OTHER"] = 303] = "SEE_OTHER";
    StatusCode[StatusCode["NOT_MODIFIED"] = 304] = "NOT_MODIFIED";
    StatusCode[StatusCode["USE_PROXY"] = 305] = "USE_PROXY";
    StatusCode[StatusCode["SWITCH_PROXY"] = 306] = "SWITCH_PROXY";
    StatusCode[StatusCode["TEMPORARY_REDIRECT"] = 307] = "TEMPORARY_REDIRECT";
    StatusCode[StatusCode["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    StatusCode[StatusCode["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
    StatusCode[StatusCode["FORBIDDEN"] = 403] = "FORBIDDEN";
    StatusCode[StatusCode["NOT_FOUND"] = 404] = "NOT_FOUND";
    StatusCode[StatusCode["METHOD_NOT_ALLOWED"] = 405] = "METHOD_NOT_ALLOWED";
    StatusCode[StatusCode["NOT_ACCEPTABLE"] = 406] = "NOT_ACCEPTABLE";
    StatusCode[StatusCode["PROXY_AUTHENTICATION_REQUIRED"] = 407] = "PROXY_AUTHENTICATION_REQUIRED";
    StatusCode[StatusCode["REQUEST_TIMEOUT"] = 408] = "REQUEST_TIMEOUT";
    StatusCode[StatusCode["CONFLICT"] = 409] = "CONFLICT";
    StatusCode[StatusCode["GONE"] = 410] = "GONE";
    StatusCode[StatusCode["LENGTH_REQUIRED"] = 411] = "LENGTH_REQUIRED";
    StatusCode[StatusCode["PRECONDITION_FAILED"] = 412] = "PRECONDITION_FAILED";
    StatusCode[StatusCode["REQUEST_ENTITY_TOO_LARGE"] = 413] = "REQUEST_ENTITY_TOO_LARGE";
    StatusCode[StatusCode["REQUEST_URI_TOO_LONG"] = 414] = "REQUEST_URI_TOO_LONG";
    StatusCode[StatusCode["UNSUPPORTED_MEDIA_TYPE"] = 415] = "UNSUPPORTED_MEDIA_TYPE";
    StatusCode[StatusCode["REQUESTED_RANGE_NOT_SATISFIABLE"] = 416] = "REQUESTED_RANGE_NOT_SATISFIABLE";
    StatusCode[StatusCode["EXPECTATION_FAILED"] = 417] = "EXPECTATION_FAILED";
    StatusCode[StatusCode["UNPROCESSABLE_ENTITY"] = 422] = "UNPROCESSABLE_ENTITY";
    StatusCode[StatusCode["LOCKED"] = 423] = "LOCKED";
    StatusCode[StatusCode["FAILED_DEPENDENCY"] = 424] = "FAILED_DEPENDENCY";
    StatusCode[StatusCode["UNORDERED_COLLECTION"] = 425] = "UNORDERED_COLLECTION";
    StatusCode[StatusCode["UPGRADE_REQUIRED"] = 426] = "UPGRADE_REQUIRED";
    StatusCode[StatusCode["RETRY_WITH"] = 449] = "RETRY_WITH";
    StatusCode[StatusCode["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
    StatusCode[StatusCode["NOT_IMPLEMENTED"] = 501] = "NOT_IMPLEMENTED";
    StatusCode[StatusCode["BAD_GATEWAY"] = 502] = "BAD_GATEWAY";
    StatusCode[StatusCode["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
    StatusCode[StatusCode["GATEWAY_TIMEOUT"] = 504] = "GATEWAY_TIMEOUT";
    StatusCode[StatusCode["HTTP_VERSION_NOT_SUPPORTED"] = 505] = "HTTP_VERSION_NOT_SUPPORTED";
    StatusCode[StatusCode["VARIANT_ALSO_NEGOTIATES"] = 506] = "VARIANT_ALSO_NEGOTIATES";
    StatusCode[StatusCode["INSUFFICIENT_STORAGE"] = 507] = "INSUFFICIENT_STORAGE";
    StatusCode[StatusCode["BANDWIDTH_LIMIT_EXCEEDED"] = 509] = "BANDWIDTH_LIMIT_EXCEEDED";
    StatusCode[StatusCode["NOT_EXTENDED"] = 510] = "NOT_EXTENDED";
    StatusCode[StatusCode["UNPARSEABLE_RESPONSE_HEADERS"] = 600] = "UNPARSEABLE_RESPONSE_HEADERS";
})(exports.StatusCode || (exports.StatusCode = {}));
var StatusCode = exports.StatusCode;
const DEFAULT_HOST = "127.0.0.1";
const DEFAULT_PORT = 80;
const DEFAULT_BACKLOG = 511;
class Server {
    constructor(opts) {
        this.serverStatus = Status.IDLE;
        this.serverOptions = {
            "host": opts !== void 0 && opts.host !== void 0 ? opts.host : DEFAULT_HOST,
            "port": opts !== void 0 && opts.port !== void 0 ? opts.port : DEFAULT_PORT,
            "backlog": opts !== void 0 && opts.backlog !== void 0 ? opts.backlog : DEFAULT_BACKLOG
        };
        this.middlewares = [];
        this.eventHandlers = {};
        this.rawURIRouters = {
            "GET": {},
            "POST": {},
            "PUT": {},
            "PATCH": {},
            "DELETE": {},
            "OPTIONS": {},
            "HEAD": {},
            "TRACE": {},
            "CONNECT": {},
            "ALL": {}
        };
        this.regexpURIRouters = {
            "GET": [],
            "POST": [],
            "PUT": [],
            "PATCH": [],
            "DELETE": [],
            "OPTIONS": [],
            "HEAD": [],
            "TRACE": [],
            "CONNECT": [],
            "ALL": []
        };
        for (let ev in Server.defaultEventHandlers) {
            this.eventHandlers[ev] = [Server.defaultEventHandlers[ev]];
        }
    }
    on(event, handler) {
        if (undefined === this.eventHandlers[event]) {
            this.eventHandlers[event] = [];
        }
        this.eventHandlers[event].push(handler);
        return this;
    }
    get backlog() {
        return this.serverOptions.backlog;
    }
    set backlog(newValue) {
        if (this.serverStatus !== Status.IDLE) {
            throw {
                "name": "BAD-STATUS",
                "message": "You can't change backlog while server is working."
            };
        }
        this.serverOptions.backlog = newValue;
    }
    get port() {
        return this.serverOptions.port;
    }
    set port(newValue) {
        if (this.serverStatus !== Status.IDLE) {
            throw {
                "name": "BAD-STATUS",
                "message": "You can't change port while server is working."
            };
        }
        this.serverOptions.port = newValue;
    }
    get host() {
        return this.serverOptions.host;
    }
    set host(newValue) {
        if (this.serverStatus !== Status.IDLE) {
            throw {
                "name": "BAD-STATUS",
                "message": "You can't change host while server is working."
            };
        }
        this.serverOptions.host = newValue;
    }
    get status() {
        return this.serverStatus;
    }
    registerStringRouter(method, routeRule, handler) {
        if (routeRule.endsWith("/") && routeRule.length > 1) {
            routeRule = routeRule.substr(0, routeRule.length - 1);
        }
        console.debug("Router: Added String-Router =", routeRule);
        this.rawURIRouters[method][routeRule] = handler;
    }
    registerStrExpRouter(method, routeRule, handler) {
        if (routeRule.endsWith("/") && routeRule.length > 1) {
            routeRule = routeRule.substr(0, routeRule.length - 1);
        }
        console.debug("Router: Added Smart-Router =", routeRule);
        let fields = [];
        routeRule = routeRule.replace(/\<.+?\>/g, function (el) {
            let rtn = "(.+?)", pos = el.indexOf(":");
            if (pos >= 0) {
                fields.push(el.substr(1, pos - 1));
                rtn = "(" + el.substring(pos + 1, el.length - 1) + ")";
            }
            else {
                fields.push(el.substr(1, el.length - 2));
            }
            return rtn;
        });
        this.regexpURIRouters[method].push({
            "exp": new RegExp("^" + routeRule + "$"),
            "fields": fields,
            "handler": handler
        });
    }
    /**
     * This method register a new handler for a routing-rule.
     */
    registerRouteHandler(method, routeRule, handler) {
        if (typeof routeRule === "string") {
            if (routeRule.match(/\<.+?\>/)) {
                this.registerStrExpRouter(method, routeRule, handler);
            }
            else {
                this.registerStringRouter(method, routeRule, handler);
            }
        }
        else if (routeRule instanceof RegExp) {
            console.debug("Router: Added Smart-Router =", routeRule);
            this.regexpURIRouters[method].push({
                "exp": routeRule,
                "handler": handler
            });
        }
    }
    all(rule, handler) {
        this.registerRouteHandler("ALL", rule, handler);
        return this;
    }
    get(rule, handler) {
        this.registerRouteHandler("GET", rule, handler);
        return this;
    }
    patch(rule, handler) {
        this.registerRouteHandler("PATCH", rule, handler);
        return this;
    }
    post(rule, handler) {
        this.registerRouteHandler("POST", rule, handler);
        return this;
    }
    options(rule, handler) {
        this.registerRouteHandler("OPTIONS", rule, handler);
        return this;
    }
    head(rule, handler) {
        this.registerRouteHandler("HEAD", rule, handler);
        return this;
    }
    put(rule, handler) {
        this.registerRouteHandler("PUT", rule, handler);
        return this;
    }
    delete(rule, handler) {
        this.registerRouteHandler("DELETE", rule, handler);
        return this;
    }
    trace(rule, handler) {
        this.registerRouteHandler("TRACE", rule, handler);
        return this;
    }
    connect(rule, handler) {
        this.registerRouteHandler("CONNECT", rule, handler);
        return this;
    }
    before(exp, middleware) {
        if (exp instanceof RegExp && middleware) {
            this.middlewares.push({
                "regExp": exp,
                "handler": middleware
            });
        }
        else if (typeof exp === "string" && middleware) {
            this.middlewares.push({
                "strExp": exp,
                "handler": middleware
            });
        }
        else {
            this.middlewares.push({
                "handler": exp
            });
        }
        return this;
    }
    start(cb) {
        if (this.serverStatus !== Status.IDLE) {
            cb({
                "name": "BAD-OPERATION",
                "message": "HTTP Server has been already started."
            });
            return this;
        }
        let self = this;
        this.serverStatus = Status.STARTING;
        this.httpServer = NodeHTTP.createServer(this.gateway.bind(this));
        this.httpServer.once("error", function (err) {
            if (err && err.syscall === "listen") {
                cb && cb({
                    "name": "LISTEN-FAILURE",
                    "message": `Failed to listen on ${self.host}:${self.port}.`
                });
            }
        });
        this.httpServer.listen(this.serverOptions.port, this.serverOptions.host, this.serverOptions.backlog, function () {
            self.serverStatus = Status.WORKING;
            self.httpServer.emit("error");
            cb && cb();
        });
        return this;
    }
    gateway(req, resp) {
        let server = this;
        if (req.path === undefined) {
            let url = NodeURL.parse(req.url);
            if (url.pathname.length > 1 && url.pathname[url.pathname.length - 1] === "/") {
                req.requestPath = url.pathname.substr(0, url.pathname.length - 1);
            }
            else {
                req.requestPath = url.pathname;
            }
            req.queryString = url.query;
            url = undefined;
            req.path = req.requestPath;
        }
        req.params = {};
        async.series([
            function (next) {
                async.forEachOf(server.middlewares, function (item, key, next) {
                    if (item.strExp && item.strExp === req.path) {
                        setImmediate(item.handler, req, resp, (err, interrupted) => {
                            if (err) {
                                return next(err);
                            }
                            next();
                        }, server);
                    }
                    else if (item.strExp === undefined && item.regExp === undefined) {
                        setImmediate(item.handler, req, resp, (err, interrupted) => {
                            if (err) {
                                return next(err);
                            }
                            next();
                        }, server);
                    }
                }, next);
            },
            function (next) {
                let handler;
                if (server.rawURIRouters[req.method] === undefined) {
                    setImmediate(server.eventHandlers["BAD-METHOD"], req, resp, next, server.rootServer ? server.rootServer : this);
                    return;
                }
                /**
                 * Lookup Text-URI handlers of specific method.
                 */
                handler = server.rawURIRouters[req.method][req.path];
                /**
                 * Lookup RegExp-URI handlers of specific method.
                 */
                if (!handler) {
                    let rules = server.regexpURIRouters[req.method];
                    for (let rule of rules) {
                        let matches;
                        if (matches = req.path.match(rule.exp)) {
                            if (rule.fields) {
                                rule.fields.forEach(function (el, index) {
                                    req.params[el] = matches[index + 1];
                                });
                            }
                            handler = rule.handler;
                            break;
                        }
                    }
                }
                /**
                 * Lookup Text-URI handlers of any method.
                 */
                if (!handler) {
                    handler = server.rawURIRouters["ALL"][req.path];
                }
                /**
                 * Lookup RegExp-URI handlers of any method.
                 */
                if (!handler) {
                    let rules = server.regexpURIRouters["ALL"];
                    for (let rule of rules) {
                        let matches;
                        if (matches = req.path.match(rule.exp)) {
                            if (rule.fields) {
                                rule.fields.forEach(function (el, index) {
                                    req.params[el] = matches[index + 1];
                                });
                            }
                            handler = rule.handler;
                            break;
                        }
                    }
                }
                if (handler) {
                    setImmediate(handler, req, resp, next, server.rootServer ? server.rootServer : this);
                }
                else {
                    next({
                        "name": "NOT-FOUND",
                        "message": null
                    });
                }
            }
        ], function (err) {
            server.trigger(err ? err.name : "COMPLETED", req, resp);
        });
    }
    trigger(eventName, req, resp) {
        if (this.eventHandlers[eventName]) {
            let server = this;
            async.forEachOf(this.eventHandlers[eventName], function (eh, key, next) {
                eh(req, resp, next, server);
            }, index_1.EmptyCallback);
        }
        else {
            Server.defaultEventHandlers[eventName] && setImmediate(Server.defaultEventHandlers[eventName], req, resp, index_1.EmptyCallback);
        }
        return this;
    }
    static init() {
        this.defaultEventHandlers = {};
        for (let eh of [
            { "id": "NOT-FOUND", "code": StatusCode.NOT_FOUND, "message": "NOT FOUND" },
            { "id": "BAD-METHOD", "code": StatusCode.METHOD_NOT_ALLOWED, "message": "METHOD NOT ALLOWED" },
            { "id": "FORBIDDEN", "code": StatusCode.FORBIDDEN, "message": "FORBIDDEN" },
            { "id": "INTERNAL", "code": StatusCode.INTERNAL_SERVER_ERROR, "message": "INTERNAL ERROR" }
        ]) {
            this.defaultEventHandlers[eh.id] = (request, response, next, server) => {
                response.statusCode = eh.code;
                response.statusMessage = eh.message;
                response.end(`HTTP ${eh.code}: ${eh.message}`);
                next();
            };
        }
        delete this.init;
    }
}
exports.Server = Server;
Server.init && Server.init();
//# sourceMappingURL=server.js.map