/// <reference path="../../typings/index.d.ts" />
"use strict";
class EventListener {
    constructor() {
        this.handlers = {};
        this.onceHandlers = {};
    }
    on(eventName, handler) {
        this.handlers[eventName] || (this.handlers[eventName] = []);
        this.handlers[eventName].push(handler);
        return this;
    }
    once(eventName, handler) {
        this.onceHandlers[eventName] || (this.onceHandlers[eventName] = []);
        this.onceHandlers[eventName].push(handler);
        return this;
    }
    unbind(eventName) {
        delete this.onceHandlers[eventName];
        delete this.handlers[eventName];
        return this;
    }
    trigger(eventName, data) {
        let handlers;
        if (handlers = this.onceHandlers[eventName]) {
            for (let handler; handler = handlers.splice(0, 1)[0];) {
                if (!handler.apply(this, [{
                        "name": eventName,
                        "data": data
                    }])) {
                    return this;
                }
            }
            delete this.onceHandlers[eventName];
        }
        if (handlers = this.handlers[eventName]) {
            for (let handler of handlers) {
                if (!handler.apply(this, [{
                        "name": eventName,
                        "data": data
                    }])) {
                    return this;
                }
            }
        }
        return this;
    }
}
exports.EventListener = EventListener;
//# sourceMappingURL=EventListener.js.map