/// <reference path="../../typings/index.d.ts" />

export interface Event {

    "name": string;

    "data"?: any;
}

export interface EventHandler {

    (e: Event): boolean;
}

export class EventListener {

    protected onceHandlers: LRTS.types.HashMap<EventHandler[]>;
    protected handlers: LRTS.types.HashMap<EventHandler[]>;

    public constructor() {

        this.handlers = {};
        this.onceHandlers = {};
    }

    public on(eventName: string, handler: EventHandler): EventListener {

        this.handlers[eventName] || (this.handlers[eventName] = []);

        this.handlers[eventName].push(handler);

        return this;
    }

    public once(eventName: string, handler: EventHandler): EventListener {

        this.onceHandlers[eventName] || (this.onceHandlers[eventName] = []);

        this.onceHandlers[eventName].push(handler);

        return this;
    }

    public unbind(eventName: string): EventListener {

        delete this.onceHandlers[eventName];
        delete this.handlers[eventName];

        return this;
    }

    public trigger(eventName: string, data?: any): EventListener {

        let handlers: EventHandler[];

        if (handlers = this.onceHandlers[eventName]) {

            for (let handler: EventHandler; handler = handlers.splice(0, 1)[0]; ) {

                if (!handler.apply(this, [{
                    "name": eventName,
                    "data": data
                }])) {

                    return this;
                }
            }

            delete this.onceHandlers[eventName];

        }

        if (handlers = this.handlers[eventName]) {

            for (let handler of handlers) {

                if (!handler.apply(this, [{
                    "name": eventName,
                    "data": data
                }])) {

                    return this;
                }
            }
        }

        return this;
    }
}
