/// <reference path="../../../typings/index.d.ts" />
"use strict";
const LoggerBase = require("./class.LoggerBase");
/**
 * The logger that write logs into console.
 */
class ConsoleLogger extends LoggerBase {
    constructor() {
        super();
        this.logWriters = {};
        this.registerType("info", function (logText, cb) {
            console.info(logText);
        });
        this.registerType("warn", function (logText, cb) {
            console.warn(logText);
        });
        this.registerType("error", function (logText, cb) {
            console.error(logText);
        });
        this.registerType("text", function (logText, cb) {
            console.log(logText);
        });
    }
    log(logText, logType, cb) {
        let writer;
        if (!logType) {
            logType = this.defType;
        }
        if (writer = this.logWriters[logType]) {
            writer(logText, cb);
        }
        else {
            cb && cb({
                "name": "BAD-LOG-TYPE",
                "message": "Specific log writer is not found."
            });
        }
    }
    logSync(logText, logType) {
        let writer;
        if (!logType) {
            logType = this.defType;
        }
        if (writer = this.logWriters[logType]) {
            writer(logText, void 0);
        }
        else {
            throw {
                "name": "BAD-LOG-TYPE",
                "message": "Specific log writer is not found."
            };
        }
    }
}
module.exports = ConsoleLogger;
//# sourceMappingURL=class.ConsoleLogger.js.map