/// <reference path="../../../typings/index.d.ts" />

import loggers = LRTS.utils.loggers;
import LoggerBase = require("./class.LoggerBase");
import { callbacks } from "lrts.globals";

/**
 * The logger that write logs into console.
 */
class ConsoleLogger extends LoggerBase implements loggers.Logger {

    public constructor() {

        super();

        this.logWriters = {};

        this.registerType("info", function(logText: any, cb?: callbacks.BooleanCallback): void {

            console.info(logText);

        });

        this.registerType("warn", function(logText: any, cb?: callbacks.BooleanCallback): void {

            console.warn(logText);

        });

        this.registerType("error", function(logText: any, cb?: callbacks.BooleanCallback): void {

            console.error(logText);

        });

        this.registerType("text", function(logText: any, cb?: callbacks.BooleanCallback): void {

            console.log(logText);

        });
    }

    public log(logText: string, logType?: string, cb?: callbacks.BooleanCallback): void {

        let writer: loggers.LogWriter;

        if (!logType) {

            logType = this.defType;
        }

        if (writer = this.logWriters[logType]) {

            writer(logText, cb);

        } else {

            cb && cb({
                "name": "BAD-LOG-TYPE",
                "message": "Specific log writer is not found."
            });
        }

    }

    public logSync(logText: string, logType?: string): void {

        let writer: loggers.LogWriter;

        if (!logType) {

            logType = this.defType;
        }

        if (writer = this.logWriters[logType]) {

            writer(logText, void 0);

        } else {

            throw {
                "name": "BAD-LOG-TYPE",
                "message": "Specific log writer is not found."
            };
        }

    }
}

export = ConsoleLogger;
